@@ -5,7 +5,8 @@ typedef enum _quoting_t {
 	QUOTE_ALL,
 	QUOTE_NONE,
 	QUOTE_MINIMAL,
-	QUOTE_NUMERIC
+	QUOTE_NUMERIC,
+	QUOTE_ORIGINAL,
 } quoting_t;
 
 #endif // QUOTING_H
