@@ -49,6 +49,7 @@ typedef struct _mapper_merge_fields_state_t {
 	sllv_t*  pvalue_field_regexes;
 	char*    output_field_basename;
 	int      allow_int_float;
+	int      do_interpolated_percentiles;
 	int      keep_input_fields;
 	string_builder_t* psb;
 } mapper_merge_fields_state_t;
@@ -57,7 +58,8 @@ typedef struct _mapper_merge_fields_state_t {
 static void      mapper_merge_fields_usage(FILE* o, char* argv0, char* verb);
 static mapper_t* mapper_merge_fields_parse_cli(int* pargi, int argc, char** argv);
 static mapper_t* mapper_merge_fields_alloc(slls_t* paccumulator_names, merge_by_t do_which,
-	slls_t* pvalue_field_names, char* output_field_basename, int allow_int_float, int keep_input_fields);
+	slls_t* pvalue_field_names, char* output_field_basename, int allow_int_float, int do_interpolated_percentiles,
+	int keep_input_fields);
 static void      mapper_merge_fields_free(mapper_t* pmapper);
 static sllv_t*   mapper_merge_fields_process_by_name_list(lrec_t* pinrec, context_t* pctx, void* pvstate);
 static sllv_t*   mapper_merge_fields_process_by_name_regex(lrec_t* pinrec, context_t* pctx, void* pvstate);
@@ -86,6 +88,7 @@ static void mapper_merge_fields_usage(FILE* o, char* argv0, char* verb) {
 	fprintf(o, "-c {a,b,c}  Substrings for collapse mode. All fields which have the same names\n");
 	fprintf(o, "            after removing substrings will be accumulated together. Please see\n");
 	fprintf(o, "            examples below.\n");
+	fprintf(o, "-i          Use interpolated percentiles, like R's type=7; default like type=1.\n");
 	fprintf(o, "-o {name}   Output field basename for -f/-r.\n");
 	fprintf(o, "-k          Keep the input fields which contributed to the output statistics;\n");
 	fprintf(o, "            the default is to omit them.\n");
@@ -103,12 +106,13 @@ static void mapper_merge_fields_usage(FILE* o, char* argv0, char* verb) {
 }
 
 static mapper_t* mapper_merge_fields_parse_cli(int* pargi, int argc, char** argv) {
-	slls_t*    paccumulator_names    = NULL;
-	slls_t*    pvalue_field_names    = NULL;
-	char*      output_field_basename = NULL;
-	int        allow_int_float       = TRUE;
-	int        keep_input_fields     = FALSE;
-	merge_by_t do_which              = MERGE_UNSPECIFIED;
+	slls_t*    paccumulator_names          = NULL;
+	slls_t*    pvalue_field_names          = NULL;
+	char*      output_field_basename       = NULL;
+	int        allow_int_float             = TRUE;
+	int        do_interpolated_percentiles = FALSE;
+	int        keep_input_fields           = FALSE;
+	merge_by_t do_which                    = MERGE_UNSPECIFIED;
 
 	char* verb = argv[(*pargi)++];
 
@@ -173,6 +177,9 @@ static mapper_t* mapper_merge_fields_parse_cli(int* pargi, int argc, char** argv
 		} else if (streq(argv[argi], "-F")) {
 			allow_int_float = FALSE;
 			argi += 1;
+		} else if (streq(argv[argi], "-i")) {
+			do_interpolated_percentiles = TRUE;
+			argi += 1;
 		} else {
 			mapper_merge_fields_usage(stderr, argv[0], verb);
 			return NULL;
@@ -197,12 +204,14 @@ static mapper_t* mapper_merge_fields_parse_cli(int* pargi, int argc, char** argv
 
 	*pargi = argi;
 	return mapper_merge_fields_alloc(paccumulator_names, do_which,
-		pvalue_field_names, output_field_basename, allow_int_float, keep_input_fields);
+		pvalue_field_names, output_field_basename, allow_int_float, do_interpolated_percentiles,
+		keep_input_fields);
 }
 
 // ----------------------------------------------------------------
 static mapper_t* mapper_merge_fields_alloc(slls_t* paccumulator_names, merge_by_t do_which,
-	slls_t* pvalue_field_names, char* output_field_basename, int allow_int_float, int keep_input_fields)
+	slls_t* pvalue_field_names, char* output_field_basename, int allow_int_float, int do_interpolated_percentiles,
+	int keep_input_fields)
 {
 	mapper_t* pmapper = mlr_malloc_or_die(sizeof(mapper_t));
 
@@ -217,10 +226,11 @@ static mapper_t* mapper_merge_fields_alloc(slls_t* paccumulator_names, merge_by_
 		regcomp_or_die(pvalue_field_regex, value_field_name, 0);
 		sllv_append(pstate->pvalue_field_regexes, pvalue_field_regex);
 	}
-	pstate->output_field_basename = output_field_basename;
-	pstate->allow_int_float       = allow_int_float;
-	pstate->keep_input_fields     = keep_input_fields;
-	pstate->psb                   = sb_alloc(SB_ALLOC_LENGTH);
+	pstate->output_field_basename       = output_field_basename;
+	pstate->allow_int_float             = allow_int_float;
+	pstate->do_interpolated_percentiles = do_interpolated_percentiles;
+	pstate->keep_input_fields           = keep_input_fields;
+	pstate->psb                         = sb_alloc(SB_ALLOC_LENGTH);
 
 	pmapper->pvstate = pstate;
 	pmapper->pprocess_func = (do_which == MERGE_BY_NAME_LIST) ? mapper_merge_fields_process_by_name_list :
@@ -252,13 +262,15 @@ static sllv_t* mapper_merge_fields_process_by_name_list(lrec_t* pinrec, context_
 		return NULL;
 
 	mapper_merge_fields_state_t* pstate = pvstate;
-	lhmsv_t* paccs = lhmsv_alloc();
-	for (sllse_t* pa = pstate->paccumulator_names->phead; pa != NULL; pa = pa->pnext) {
-		char* acc_name = pa->value;
-		stats1_acc_t* pacc = make_stats1_acc(pstate->output_field_basename, acc_name,
-			pstate->allow_int_float);
-		lhmsv_put(paccs, acc_name, pacc, NO_FREE);
-	}
+	// For percentiles there is one unique accumulator given (for example) five distinct
+	// names p0,p25,p50,p75,p100.  The input accumulators are unique: only one
+	// percentile-keeper. There are multiple output accumulators: each references the same
+	// underlying percentile-keeper but with distinct parameters.
+	lhmsv_t* pinaccs = lhmsv_alloc();
+	lhmsv_t* poutaccs = lhmsv_alloc();
+
+	make_stats1_accs(pstate->output_field_basename, pstate->paccumulator_names,
+	    pstate->allow_int_float, pstate->do_interpolated_percentiles, pinaccs, poutaccs);
 
 	for (sllse_t* pb = pstate->pvalue_field_names->phead; pb != NULL; pb = pb->pnext) {
 		char* field_name = pb->value;
@@ -275,9 +287,9 @@ static sllv_t* mapper_merge_fields_process_by_name_list(lrec_t* pinrec, context_
 		int have_dval = FALSE;
 		int have_nval = FALSE;
 		double value_field_dval = -999.0;
-		mv_t   value_field_nval = mv_from_null();
+		mv_t   value_field_nval = mv_absent();
 
-		for (lhmsve_t* pc = paccs->phead; pc != NULL; pc = pc->pnext) {
+		for (lhmsve_t* pc = pinaccs->phead; pc != NULL; pc = pc->pnext) {
 			stats1_acc_t* pacc = pc->pvvalue;
 
 			if (pacc->pdingest_func != NULL) {
@@ -305,13 +317,14 @@ static sllv_t* mapper_merge_fields_process_by_name_list(lrec_t* pinrec, context_
 			lrec_remove(pinrec, field_name);
 	}
 
-	for (lhmsve_t* pz = paccs->phead; pz != NULL; pz = pz->pnext) {
+	for (lhmsve_t* pz = poutaccs->phead; pz != NULL; pz = pz->pnext) {
 		char* acc_name = pz->key;
 		stats1_acc_t* pacc = pz->pvvalue;
 		pacc->pemit_func(pacc->pvstate, pstate->output_field_basename, acc_name, TRUE, pinrec);
 		pacc->pfree_func(pacc);
 	}
-	lhmsv_free(paccs);
+	lhmsv_free(pinaccs);
+	lhmsv_free(poutaccs);
 
 	return sllv_single(pinrec);
 }
@@ -322,13 +335,17 @@ static sllv_t* mapper_merge_fields_process_by_name_regex(lrec_t* pinrec, context
 		return NULL;
 
 	mapper_merge_fields_state_t* pstate = pvstate;
-	lhmsv_t* paccs = lhmsv_alloc();
-	for (sllse_t* pa = pstate->paccumulator_names->phead; pa != NULL; pa = pa->pnext) {
-		char* acc_name = pa->value;
-		stats1_acc_t* pacc = make_stats1_acc(pstate->output_field_basename, acc_name,
-			pstate->allow_int_float);
-		lhmsv_put(paccs, acc_name, pacc, NO_FREE);
-	}
+
+	// For percentiles there is one unique accumulator given (for example) five distinct
+	// names p0,p25,p50,p75,p100.  The input accumulators are unique: only one
+	// percentile-keeper. There are multiple output accumulators: each references the same
+	// underlying percentile-keeper but with distinct parameters.
+
+	lhmsv_t* pinaccs = lhmsv_alloc();
+	lhmsv_t* poutaccs = lhmsv_alloc();
+
+	make_stats1_accs(pstate->output_field_basename, pstate->paccumulator_names,
+	    pstate->allow_int_float, pstate->do_interpolated_percentiles, pinaccs, poutaccs);
 
 	for (lrece_t* pb = pinrec->phead; pb != NULL; /* increment inside loop */ ) {
 		char* field_name = pb->key;
@@ -342,10 +359,10 @@ static sllv_t* mapper_merge_fields_process_by_name_regex(lrec_t* pinrec, context
 					int have_dval = FALSE;
 					int have_nval = FALSE;
 					double value_field_dval = -999.0;
-					mv_t   value_field_nval = mv_from_null();
+					mv_t   value_field_nval = mv_absent();
 
 					if (*value_field_sval != 0) { // Key present with null value
-						for (lhmsve_t* pd = paccs->phead; pd != NULL; pd = pd->pnext) {
+						for (lhmsve_t* pd = pinaccs->phead; pd != NULL; pd = pd->pnext) {
 							stats1_acc_t* pacc = pd->pvvalue;
 
 							if (pacc->pdingest_func != NULL) {
@@ -387,13 +404,14 @@ static sllv_t* mapper_merge_fields_process_by_name_regex(lrec_t* pinrec, context
 			pb = pb->pnext;
 	}
 
-	for (lhmsve_t* pz = paccs->phead; pz != NULL; pz = pz->pnext) {
+	for (lhmsve_t* pz = poutaccs->phead; pz != NULL; pz = pz->pnext) {
 		char* acc_name = pz->key;
 		stats1_acc_t* pacc = pz->pvvalue;
 		pacc->pemit_func(pacc->pvstate, pstate->output_field_basename, acc_name, TRUE, pinrec);
 		pacc->pfree_func(pacc);
 	}
-	lhmsv_free(paccs);
+	lhmsv_free(pinaccs);
+	lhmsv_free(poutaccs);
 
 	return sllv_single(pinrec);
 }
@@ -410,7 +428,14 @@ static sllv_t* mapper_merge_fields_process_by_collapsing(lrec_t* pinrec, context
 		return NULL;
 
 	mapper_merge_fields_state_t* pstate = pvstate;
-	lhmsv_t* short_names_to_acc_maps = lhmsv_alloc();
+
+	// For percentiles there is one unique accumulator given (for example) five distinct
+	// names p0,p25,p50,p75,p100.  The input accumulators are unique: only one
+	// percentile-keeper. There are multiple output accumulators: each references the same
+	// underlying percentile-keeper but with distinct parameters.
+
+	lhmsv_t* short_names_to_in_acc_maps = lhmsv_alloc();
+	lhmsv_t* short_names_to_out_acc_maps = lhmsv_alloc();
 
 	for (lrece_t* pa = pinrec->phead; pa != NULL; /* increment inside loop */ ) {
 		char* field_name = pa->key;
@@ -419,29 +444,35 @@ static sllv_t* mapper_merge_fields_process_by_collapsing(lrec_t* pinrec, context
 			regex_t* pvalue_field_regex = pb->pvvalue;
 			char* short_name = regex_sub(field_name, pvalue_field_regex, pstate->psb, "", &matched, NULL);
 			if (matched) {
-				lhmsv_t* acc_map_for_short_name = lhmsv_get(short_names_to_acc_maps, short_name);
-				if (acc_map_for_short_name == NULL) { // First such
-					acc_map_for_short_name = lhmsv_alloc();
-					for (sllse_t* pc = pstate->paccumulator_names->phead; pc != NULL; pc = pc->pnext) {
-						char* acc_name = pc->value;
-						stats1_acc_t* pacc = make_stats1_acc(short_name, acc_name, pstate->allow_int_float);
-						lhmsv_put(acc_map_for_short_name, acc_name, pacc, NO_FREE);
-					}
-					lhmsv_put(short_names_to_acc_maps, mlr_strdup_or_die(short_name), acc_map_for_short_name,
+				lhmsv_t* in_acc_map_for_short_name = lhmsv_get(short_names_to_in_acc_maps, short_name);
+				lhmsv_t* out_acc_map_for_short_name = lhmsv_get(short_names_to_out_acc_maps, short_name);
+				if (out_acc_map_for_short_name == NULL) { // First such
+
+					in_acc_map_for_short_name = lhmsv_alloc();
+					out_acc_map_for_short_name = lhmsv_alloc();
+
+					make_stats1_accs(short_name, pstate->paccumulator_names,
+						pstate->allow_int_float, pstate->do_interpolated_percentiles,
+						in_acc_map_for_short_name, out_acc_map_for_short_name);
+
+					lhmsv_put(short_names_to_in_acc_maps, mlr_strdup_or_die(short_name), in_acc_map_for_short_name,
 						FREE_ENTRY_KEY);
+					lhmsv_put(short_names_to_out_acc_maps, mlr_strdup_or_die(short_name), out_acc_map_for_short_name,
+						FREE_ENTRY_KEY);
+
 				}
 
 				char* value_field_sval = lrec_get(pinrec, field_name);
 				if (value_field_sval != NULL) { // Key present
 
 					if (*value_field_sval != 0) { // Key present with non-null value
-						for (lhmsve_t* pd = acc_map_for_short_name->phead; pd != NULL; pd = pd->pnext) {
+						for (lhmsve_t* pd = in_acc_map_for_short_name->phead; pd != NULL; pd = pd->pnext) {
 							stats1_acc_t* pacc = pd->pvvalue;
 
 							int have_dval = FALSE;
 							int have_nval = FALSE;
 							double value_field_dval = -999.0;
-							mv_t   value_field_nval = mv_from_null();
+							mv_t   value_field_nval = mv_absent();
 
 							if (pacc->pdingest_func != NULL) {
 								if (!have_dval) {
@@ -483,25 +514,31 @@ static sllv_t* mapper_merge_fields_process_by_collapsing(lrec_t* pinrec, context
 			pa = pa->pnext;
 	}
 
-	for (lhmsve_t* pe = short_names_to_acc_maps->phead; pe != NULL; pe = pe->pnext) {
+	for (lhmsve_t* pe = short_names_to_out_acc_maps->phead; pe != NULL; pe = pe->pnext) {
 		char* short_name = pe->key;
-		lhmsv_t* acc_map_for_short_name = pe->pvvalue;
-		for (lhmsve_t* pf = acc_map_for_short_name->phead; pf != NULL; pf = pf->pnext) {
+		lhmsv_t* out_acc_map_for_short_name = pe->pvvalue;
+		for (lhmsve_t* pf = out_acc_map_for_short_name->phead; pf != NULL; pf = pf->pnext) {
 			char* acc_name = pf->key;
 			stats1_acc_t* pacc = pf->pvvalue;
 			pacc->pemit_func(pacc->pvstate, short_name, acc_name, TRUE, pinrec);
 		}
 	}
 
-	for (lhmsve_t* pe = short_names_to_acc_maps->phead; pe != NULL; pe = pe->pnext) {
-		lhmsv_t* acc_map_for_short_name = pe->pvvalue;
-		for (lhmsve_t* pf = acc_map_for_short_name->phead; pf != NULL; pf = pf->pnext) {
+	for (lhmsve_t* pe = short_names_to_out_acc_maps->phead; pe != NULL; pe = pe->pnext) {
+		lhmsv_t* out_acc_map_for_short_name = pe->pvvalue;
+		for (lhmsve_t* pf = out_acc_map_for_short_name->phead; pf != NULL; pf = pf->pnext) {
 			stats1_acc_t* pacc = pf->pvvalue;
 			pacc->pfree_func(pacc);
 		}
-		lhmsv_free(acc_map_for_short_name);
+		lhmsv_free(out_acc_map_for_short_name);
 	}
 
-	lhmsv_free(short_names_to_acc_maps);
+	for (lhmsve_t* pe = short_names_to_in_acc_maps->phead; pe != NULL; pe = pe->pnext) {
+		lhmsv_t* in_acc_map_for_short_name = pe->pvvalue;
+		lhmsv_free(in_acc_map_for_short_name);
+	}
+
+	lhmsv_free(short_names_to_in_acc_maps);
+	lhmsv_free(short_names_to_out_acc_maps);
 	return sllv_single(pinrec);
 }
