@@ -80,6 +80,7 @@ static void      mapper_step_free(mapper_t* pmapper);
 static sllv_t*   mapper_step_process(lrec_t* pinrec, context_t* pctx, void* pvstate);
 
 static step_t* step_delta_alloc      (char* input_field_name, int allow_int_float, slls_t* unused1, slls_t* unused2);
+static step_t* step_shift_alloc      (char* input_field_name, int allow_int_float, slls_t* unused1, slls_t* unused2);
 static step_t* step_from_first_alloc (char* input_field_name, int allow_int_float, slls_t* unused1, slls_t* unused2);
 static step_t* step_ratio_alloc      (char* input_field_name, int allow_int_float, slls_t* unused1, slls_t* unused2);
 static step_t* step_rsum_alloc       (char* input_field_name, int allow_int_float, slls_t* unused1, slls_t* unused2);
@@ -97,6 +98,7 @@ typedef struct _step_lookup_t {
 } step_lookup_t;
 static step_lookup_t step_lookup_table[] = {
 	{"delta",      step_delta_alloc,      "Compute differences in field(s) between successive records"},
+	{"shift",      step_shift_alloc,      "Include value(s) in field(s) from previous record, if any"},
 	{"from-first", step_from_first_alloc, "Compute differences in field(s) from first record"},
 	{"ratio",      step_ratio_alloc,      "Compute ratios in field(s) between successive records"},
 	{"rsum",       step_rsum_alloc,       "Compute running sums of field(s) between successive records"},
@@ -272,7 +274,7 @@ static sllv_t* mapper_step_process(lrec_t* pinrec, context_t* pctx, void* pvstat
 		int have_dval = FALSE;
 		int have_nval = FALSE;
 		double value_field_dval = -999.0;
-		mv_t   value_field_nval = mv_from_null();
+		mv_t   value_field_nval = mv_absent();
 
 		lhmsv_t* pacc_field_to_acc_state = lhmsv_get(pgroup_to_acc_field, value_field_name);
 		if (pacc_field_to_acc_state == NULL) {
@@ -351,7 +353,7 @@ static void step_delta_nprocess(void* pvstate, mv_t* pnumv, lrec_t* prec) {
 	if (mv_is_null(&pstate->prev)) {
 		delta = pstate->allow_int_float ? mv_from_int(0LL) : mv_from_float(0.0);
 	} else {
-		delta = n_nn_minus_func(pnumv, &pstate->prev);
+		delta = x_xx_minus_func(pnumv, &pstate->prev);
 	}
 	lrec_put(prec, pstate->output_field_name, mv_alloc_format_val(&delta), FREE_ENTRY_VALUE);
 	pstate->prev = *pnumv;
@@ -369,7 +371,7 @@ static void step_delta_free(step_t* pstep) {
 static step_t* step_delta_alloc(char* input_field_name, int allow_int_float, slls_t* unused1, slls_t* unused2) {
 	step_t* pstep = mlr_malloc_or_die(sizeof(step_t));
 	step_delta_state_t* pstate = mlr_malloc_or_die(sizeof(step_delta_state_t));
-	pstate->prev = mv_from_null();
+	pstate->prev = mv_absent();
 	pstate->allow_int_float = allow_int_float;
 	pstate->output_field_name = mlr_paste_2_strings(input_field_name, "_delta");
 	pstep->pvstate        = (void*)pstate;
@@ -381,6 +383,43 @@ static step_t* step_delta_alloc(char* input_field_name, int allow_int_float, sll
 	return pstep;
 }
 
+// ----------------------------------------------------------------
+typedef struct _step_shift_state_t {
+	char* prev;
+	char* output_field_name;
+	int   allow_int_float;
+} step_shift_state_t;
+static void step_shift_sprocess(void* pvstate, char* strv, lrec_t* prec) {
+	step_shift_state_t* pstate = pvstate;
+	lrec_put(prec, pstate->output_field_name, pstate->prev, FREE_ENTRY_VALUE);
+	pstate->prev = mlr_strdup_or_die(strv);
+}
+static void step_shift_zprocess(void* pvstate, lrec_t* prec) {
+	step_shift_state_t* pstate = pvstate;
+	lrec_put(prec, pstate->output_field_name, "", NO_FREE);
+}
+static void step_shift_free(step_t* pstep) {
+	step_shift_state_t* pstate = pstep->pvstate;
+	free(pstate->prev);
+	free(pstate->output_field_name);
+	free(pstate);
+	free(pstep);
+}
+static step_t* step_shift_alloc(char* input_field_name, int allow_int_float, slls_t* unused1, slls_t* unused2) {
+	step_t* pstep = mlr_malloc_or_die(sizeof(step_t));
+	step_shift_state_t* pstate = mlr_malloc_or_die(sizeof(step_shift_state_t));
+	pstate->prev = mlr_strdup_or_die("");
+	pstate->allow_int_float = allow_int_float;
+	pstate->output_field_name = mlr_paste_2_strings(input_field_name, "_shift");
+	pstep->pvstate        = (void*)pstate;
+	pstep->pdprocess_func = NULL;
+	pstep->pnprocess_func = NULL;
+	pstep->psprocess_func = step_shift_sprocess;
+	pstep->pzprocess_func = step_shift_zprocess;
+	pstep->pfree_func     = step_shift_free;
+	return pstep;
+}
+
 // ----------------------------------------------------------------
 typedef struct _step_from_first_state_t {
 	mv_t  first;
@@ -394,7 +433,7 @@ static void step_from_first_nprocess(void* pvstate, mv_t* pnumv, lrec_t* prec) {
 		from_first = pstate->allow_int_float ? mv_from_int(0LL) : mv_from_float(0.0);
 		pstate->first = *pnumv;
 	} else {
-		from_first = n_nn_minus_func(pnumv, &pstate->first);
+		from_first = x_xx_minus_func(pnumv, &pstate->first);
 	}
 	lrec_put(prec, pstate->output_field_name, mv_alloc_format_val(&from_first), FREE_ENTRY_VALUE);
 }
@@ -411,7 +450,7 @@ static void step_from_first_free(step_t* pstep) {
 static step_t* step_from_first_alloc(char* input_field_name, int allow_int_float, slls_t* unused1, slls_t* unused2) {
 	step_t* pstep = mlr_malloc_or_die(sizeof(step_t));
 	step_from_first_state_t* pstate = mlr_malloc_or_die(sizeof(step_from_first_state_t));
-	pstate->first = mv_from_null();
+	pstate->first = mv_absent();
 	pstate->allow_int_float = allow_int_float;
 	pstate->output_field_name = mlr_paste_2_strings(input_field_name, "_from_first");
 	pstep->pvstate        = (void*)pstate;
@@ -475,7 +514,7 @@ typedef struct _step_rsum_state_t {
 } step_rsum_state_t;
 static void step_rsum_nprocess(void* pvstate, mv_t* pnumv, lrec_t* prec) {
 	step_rsum_state_t* pstate = pvstate;
-	pstate->rsum = n_nn_plus_func(&pstate->rsum, pnumv);
+	pstate->rsum = x_xx_plus_func(&pstate->rsum, pnumv);
 	lrec_put(prec, pstate->output_field_name, mv_alloc_format_val(&pstate->rsum),
 		FREE_ENTRY_VALUE);
 }
@@ -512,7 +551,7 @@ typedef struct _step_counter_state_t {
 } step_counter_state_t;
 static void step_counter_sprocess(void* pvstate, char* strv, lrec_t* prec) {
 	step_counter_state_t* pstate = pvstate;
-	pstate->counter = n_nn_plus_func(&pstate->counter, &pstate->one);
+	pstate->counter = x_xx_plus_func(&pstate->counter, &pstate->one);
 	lrec_put(prec, pstate->output_field_name, mv_alloc_format_val(&pstate->counter),
 		FREE_ENTRY_VALUE);
 }
