@@ -18,13 +18,14 @@ int main(int argc, char** argv) {
 	cli_opts_t* popts = parse_command_line(argc, argv);
 	mlr_global_init(argv[0], popts->ofmt, popts);
 
-	char*          prepipe      = popts->prepipe;
+	char*          prepipe      = popts->reader_opts.prepipe;
 	lrec_reader_t* plrec_reader = popts->plrec_reader;
 	sllv_t*        pmapper_list = popts->pmapper_list;
 	lrec_writer_t* plrec_writer = popts->plrec_writer;
-	char**         filenames    = popts->filenames;
+	slls_t*        filenames    = popts->filenames;
 
-	int ok = do_stream_chained(prepipe, filenames, plrec_reader, pmapper_list, plrec_writer, popts->ofmt);
+	int ok = do_stream_chained(prepipe, filenames, plrec_reader, pmapper_list, plrec_writer, popts->ofmt,
+		popts->nr_progress_mod);
 
 	cli_opts_free(popts);
 
